#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

from __future__ import with_statement

import os

from ionas.exception import *


class FileLikeObj():

    def __init__(self, file, path):

        self.file = file
        self.path = path
        self.point = 0

    def __enter__(self):

        return self

    def __exit__(self, exec_type, exec_val, exec_tb):

        self.close()
        return False

    def __len__(self):

        raise NotImplementedError

    def tell(self):

        raise NotImplementedError

    def get_path(self):

        return self.path

    def read(self):

        raise NotImplementedError

    def write(self, file_like):

        raise NotImplementedError

    def write_by_path(self, file_like):

        raise NotImplementedError

    def read_by_path(self, file_like):

        raise NotImplementedError

    def close(self):

        raise NotImplementedError


class FilePart:

    def __init__(self, file_like, offset=0, bytes=None):

        self.file = file_like.file

        self.offset = offset
        self.bytes = bytes
        if bytes is None:
            self.bytes = file_like.get_size()

        self.seek(0)

    def seek(self, offset, whence=os.SEEK_SET):

        if whence == os.SEEK_SET:
            self.file.seek(self.offset + offset)
        elif whence == os.SEEK_CUR:
            self.file.seek(self.tell() + offset)
        elif whence == os.SEEK_END:
            self.file.seek(self.bytes + offset)

    def tell(self):

        return self.file.tell() - self.offset

    def read(self, size=-1):

        if size >= 0:
            max_size = self.bytes - self.tell()
            size = min([size, max_size])
            return self.file.read(size)
        else:
            return self.readall()

    def readall(self):

        return self.file.read(self.bytes - self.tell())
